"""
REST API for our application
"""
import json
from aiohttp import web

from unit_registry import AbstractUnitRegistry
from message_publisher import AbstractMessagePublisher


def get_address(request):
    """
    Extract the address parameter from request.

    Raise HTTP Bad Request if address parameter is missing
    """
    parameters = request.rel_url.query
    address = ""
    try:
        address = parameters['address']
    except KeyError:
        raise web.HTTPBadRequest(text="Missing address parameter")

    if not address.startswith("tunnel-"):
        address = "tunnel-{}".format(address)
    return address


class ApplicationHandler(web.Application):
    """
    Main class representing a HTTP application.
    """

    def __init__(self, document_storage: AbstractUnitRegistry,
                 message_publisher: AbstractMessagePublisher):
        super().__init__()
        base_url = "/api/v1/targets"
        self.document_storage = document_storage
        self.message_publisher = message_publisher
        self.router.add_get(base_url, self.handle_get)
        self.router.add_post(base_url, self.handle_post)
        self.router.add_delete(base_url, self.handle_delete)

    async def handle_get(self, _):
        """ What should happen when accessed through HTTP GET """
        registered_units = await self.document_storage.get_registered_units()
        return web.Response(body=json.dumps(registered_units).encode())

    async def handle_post(self, request):
        """ What should happen when accessed through HTTP POST """
        address = get_address(request)
        registered_units = await self.document_storage.get_registered_units()

        if address in registered_units:
            bdy = "{} already registered".format(address).encode()
            return web.Response(status=400, body=bdy)

        await self.document_storage.register_unit(address)
        await self.message_publisher.publish(address)

        return web.Response(
            body='{} registered successfully'.format(address).encode())

    async def handle_delete(self, request):
        """ What should happen when accessed through HTTP DELETE """
        address = get_address(request)
        registered_units = await self.document_storage.get_registered_units()

        if address not in registered_units:
            return web.Response(status=400,
                                body="No such device registered".encode())
        else:
            await self.document_storage.unregister_unit(address)
            return web.Response(status=200,
                                body="{} unregistered successfully".format(
                                    address).encode())
