"""
Unit tests for our REST API
"""
from unittest.mock import MagicMock
from aiohttp.test_utils import AioHTTPTestCase, unittest_run_loop

from application import ApplicationHandler

from unit_registry import MockUnitRegistry
from message_publisher import MockMessagePublisher


class MyAppTestCase(AioHTTPTestCase):
    """ Test case """

    async def get_application(self):
        """
        Override the get_app method to return your application.
        """
        self.supermock = MagicMock()
        self.unit_registry = MockUnitRegistry()
        self.mock_message_publisher = MockMessagePublisher()
        self.message_publisher = self.supermock.message_publisher
        self.doc_storage = self.supermock.doc_storage

        tmp = MagicMock(side_effect=self.unit_registry.get_registry)
        self.doc_storage.get_registered_units = tmp
        tmp = MagicMock(side_effect=self.unit_registry.register)
        self.doc_storage.register_unit = tmp
        tmp = MagicMock(side_effect=self.unit_registry.unregister)
        self.doc_storage.unregister_unit = tmp

        tmp = MagicMock(side_effect=self.mock_message_publisher.publish)
        self.message_publisher.publish = tmp

        return ApplicationHandler(self.doc_storage, self.message_publisher)

    @unittest_run_loop
    async def test_get_nothing_registered(self):
        """
        Define what should happen when we perform a HTTP GET while no units are
        registered
        """
        request = await self.client.request("GET", "/api/v1/targets")
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("[]", text)
        self.doc_storage.get_registered_units.assert_called_once_with()
        self.doc_storage.register_unit.assert_not_called()
        self.message_publisher.publish.assert_not_called()

    @unittest_run_loop
    async def test_get_unit_registered(self):
        """
        Define what should happen when we perform a HTTP GET with one unit
        registered
        """
        unit = "123456"
        path = "/api/v1/targets?address={}".format(unit)
        request = await self.client.request("POST", path)
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("tunnel-{} registered successfully".format(unit),
                         text)
        request = await self.client.request("GET", "/api/v1/targets")
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("[\"tunnel-{}\"]".format(unit), text)

    @unittest_run_loop
    async def test_missing_address(self):
        """
        Specify what should happen when we miss to supply the address parameter
        """
        path = "/api/v1/targets"
        methods_require_address = ['POST', 'DELETE']
        for method in methods_require_address:
            request = await self.client.request(method, path)
            self.assertEqual(request.status, 400)
            text = await request.text()
            self.assertEqual("Missing address parameter", text)

    @unittest_run_loop
    async def test_register_multiple_unit(self):
        """
        Define what should happen when registering multiple units
        """
        units = ['123', 'tunnel-456', '789']

        for unit in units:
            path = "/api/v1/targets?address={}".format(unit)
            request = await self.client.request("POST", path)
            self.assertEqual(request.status, 200)
            text = await request.text()
            if not unit.startswith("tunnel-"):
                unit = "tunnel-{}".format(unit)
            self.assertEqual("{} registered successfully".format(unit),
                             text)
            self.doc_storage.register_unit.assert_called_with(unit)
            self.message_publisher.publish.assert_called_with(unit)
        calls = list(map(str, self.supermock.method_calls))
        before = "call.doc_storage.register_unit('{}')"
        after = "call.message_publisher.publish('{}')"
        for unit in units:
            if not unit.startswith("tunnel-"):
                unit = "tunnel-{}".format(unit)
            self.assertGreater(calls.index(after.format(unit)),
                               calls.index(before.format(unit)))

    @unittest_run_loop
    async def test_re_register(self):
        """
        Specify what should happen when trying to register an already
        registered unit
        """
        unit = "123456"
        path = "/api/v1/targets?address={}".format(unit)
        request = await self.client.request("POST", path)
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("tunnel-{} registered successfully".format(unit),
                         text)

        request = await self.client.request("POST", path)
        self.assertEqual(request.status, 400)
        text = await request.text()
        self.assertEqual("tunnel-{} already registered".format(unit), text)

    @unittest_run_loop
    async def test_remove_unregistered_unit(self):
        """
        Define what should happen when trying to unregister a unit that was
        never registered
        """
        path = "/api/v1/targets?address=tunnel-123"
        request = await self.client.request("DELETE", path)
        self.assertEqual(request.status, 400)
        text = await request.text()
        self.assertEqual("No such device registered", text)
        self.doc_storage.register_unit.assert_not_called()
        self.message_publisher.publish.assert_not_called()

    @unittest_run_loop
    async def test_remove_registered_unit(self):
        """ Specifu what happens when we unregister a registered unit """
        unit = "123456"
        path = "/api/v1/targets?address={}".format(unit)
        request = await self.client.request("POST", path)
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("tunnel-{} registered successfully".format(unit),
                         text)

        request = await self.client.request("GET", "/api/v1/targets")
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("[\"tunnel-{}\"]".format(unit), text)

        path = "/api/v1/targets?address={}".format(unit)
        request = await self.client.request("DELETE", path)
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("tunnel-{} unregistered successfully".format(unit),
                         text)

        request = await self.client.request("GET", "/api/v1/targets")
        self.assertEqual(request.status, 200)
        text = await request.text()
        self.assertEqual("[]", text)
